/*************************************************************/
/*                                                           */
/* mandel.c                                                  */
/*                                                           */
/* Mandelbrot fractal program.                               */
/*                                                           */
/* By Stefan Spaennare, Lund, Sweden                         */
/*                                                           */
/* E-mail: stefan@spaennare.se                               */
/*                                                           */
/* First version: July 1992                                  */
/* Latest update: 2006-02-06                                 */
/*                                                           */
/*************************************************************/

/****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "image.h"


double timetic=(double)(CLOCKS_PER_SEC);


int iter(c1,c2,itmax)
double c1,c2;
int itmax;
{
   double z1,z2,z3;
   int it;

   z1=0.0;
   z2=0.0;
   it=0;

   while ((it < itmax) && ((z1*z1+z2*z2) <= 4.0)) {
      z3=z1;
      z1=z1*z1-z2*z2+c1;
      z2=2.0*z3*z2+c2;
      it++;
   } 
   return(it);
} /* end iter */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *infile;

   struct image_head header;    
   short **image;

   double a,b,da,db,ca,cb,wa,wb,flops,ftime;
   int i,j,itmax,it,xsize,ysize;
   clock_t time1,timediff;


   if (argc != 3) {
      printf("Usage: %s image mandfile\n",argv[0]);
      printf("\n");
      printf("Parameters: a wa b wb itmax xsize ysize\n"); 
      exit(0);
   } /* if */


   if ((infile = fopen(argv[2],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */


   fscanf(infile,"%lf %lf %lf %lf %d %d %d",&a,&wa,&b,&wb,&itmax,&xsize,&ysize); 

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=ysize;
   header.wid=xsize;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);



da=wa/(double)(xsize);
db=wb/(double)(ysize);

printf("\n");
printf("Mandelbrot fractal\n");
printf("\n");
printf("itmax= %6d; xsize= %6d; ysize= %6d\n",itmax,xsize,ysize);
printf("a= %18.16f; wa= %18.16f\n",a,wa);
printf("b= %18.16f; wb= %18.16f\n",b,wb);
printf("\n");


flops=0.0;

time1=clock();

for (i=0; i<ysize; i++) {
   if ((i % 100) == 0)
      printf("%4d\n",i); 
   for (j=0; j<xsize; j++) {
      ca=a+da*(double)(j);
      cb=b+db*(double)(ysize-i-1);
      it=iter(ca,cb,itmax); 
      image[i][j]=it;
      flops=flops+10.0*(double)(it)+7.0;
   } /* for j */
} /* for i */


timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
flops=flops/ftime;

printf("\n");
printf("CPU-time: %9.2f s\n",ftime);
printf("Mflops:   %9.2f\n",flops/1.0e6);
printf("\n");

WriteImage(argv[1],image,header,im_s16);

fclose(infile);

} /* End */
